use super::ast::*;
use crate::lex::tok::*;
use logos::Lexer;
use std::rc::Rc;

#[macro_export]
macro_rules! unwrap {
    ($var:expr) => {
	$var.next().unwrap().unwrap()
    }
}

pub fn expect_number(n: Option<Result<Token, ()>>) -> Expr<'static> {
    match n {
	Some(Ok(Number(res))) => Expr::Number(res),
	Some(Ok(tok)) => {
	    println!("Expected a number, got: {}", tok);
	    std::process::exit(1);
	}
	_ => {
	    println!("Expected a number, got EOF");
	    std::process::exit(1);
	}
    }
}

pub fn parse_math(mut tokens: Lexer<Token>) -> Option<Expr> {
    let mut left = expect_number(tokens.next());
    
    loop {
	let op = tokens.next();
	if !match_operator(&op).is_some() { break }
	let right = expect_number(tokens.next());
	left = Expr::MathExpr(Math {left: Rc::new(left), right: Rc::new(right), operator: match_operator(&op)?});
    }
    Some(left)
}

pub fn match_operator(token: &Option<Result<Token, ()>>) -> Option<MathOperator> {
    if let Some(Ok(token)) = token {
	return match token {
	    Plus    => Some(MathOperator::OP_ADD),
	    Minus   => Some(MathOperator::OP_SUB),
	    Slash   => Some(MathOperator::OP_DIV),
	    Star    => Some(MathOperator::OP_MULT),
 	    Percent => Some(MathOperator::OP_MOD),
	    _ => None
	};
    }
    
    None
}
